/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : April, 1997                                                    */
/* File Name : SetCoverSolution.h                                        */
/*************************************************************************/
/*************************************************************************/

#ifndef _SETCOVERSOLUTION_H
#define _SETCOVERSOLUTION_H

#include <iostream.h>

#include <deffunc.h>
#include <tSet.h>
#include <SortedSet.h>

#include "Column.h"
#include "SetCoverProblem.h"

class SetCoverSolution
{ 
  public:

     enum GREEDY_TYPE { RANDOM, BACK_GRDY, CHAVATAL_GRDY, GRDY_3 };

     SetCoverSolution(SetCoverProblem *);
     SetCoverSolution();
     SetCoverSolution(const SetCoverSolution&);
     ~SetCoverSolution();

     SetCoverProblem * getProblem() const ;

     Column* getColumnWithIndx(int) const;

     void addColumn(size_t columnIndex);

     void deleteColumnWithIndex(size_t);
     tSet<int> getColumns(); 

     double getCost() const;

     unsigned getNumOfColumns() const;
     unsigned getNumberOfOverlaps() const;

     getColNumWithIndex(int) const;

     getNumOfNonZerosUpTo(int);

     char *getCover() const;
     char *getCover(int) const;
     int  *getWeights() const { return weights; };

     char   *getCoverWithoutColIndex(int);
     Column getCoverColumn(int indx=-1) const;

     void organize();
     void solveRandom(int);
     void setProblem(SetCoverProblem *);
     void restore(int start);
     restoreRandomChoice(tSet<int>* nlegal=NULL);
     restoreGreedyChoice(tSet<int>* nlegal=NULL);

     restoreRandom(tSet<int>* nlegal=NULL);
     restoreBack(tSet<int>* nlegal=NULL);
     restoreChavatal(tSet<int>* nlegal=NULL);
     restoreGreedy3(tSet<int>* nlegal=NULL);

       // Picks a column from set R1 according to the given algorithm
       // type.
     pickChoice(Column * cover, tSet<int>* R1, GREEDY_TYPE gType=RANDOM);

     void solveWithGreedy(GREEDY_TYPE gType = BACK_GRDY);
     void solveWithRandomChoice(int start=0);

     SetCoverSolution operator=(const SetCoverSolution&);
     operator==(const SetCoverSolution&);

     void printColumnsBinary(ostream&) const ;
     void printColumnsBinary() const { printColumnsBinary(cout); };
     void printColumns(ostream&) const ;
     void printColumns() const { printColumns(cout); };

     friend ostream &operator<<(ostream&,const SetCoverSolution&);
     void reduce();

  protected:

  SetCoverProblem *myProblem;
   SortedSet<int,int> Solution; 
      //tSet<int> Solution; 
      double    cost;
      Column*   myCover;
      int*      weights;
	  

     void copyFrom(const SetCoverSolution& RHS);
     void dispose();
     Column* findMinimumCostRatio(Column* U);
     Column* findMinimumCostRatioCHAVATAL(Column* U);
     Column* findMinimumCostRatioGRDY3(Column* U);

       // Restoration according to abondoned Column indices and the
       // algorithm type.
     restoreG(tSet<int>* nlegal, GREEDY_TYPE gType);

/**

     Solution keeps the set of column indices.
*/

}; // end of SetCoverSolution

#endif
